#!/usr/bin/env python
# -*- coding: utf-8 -*-

# sample_color_border.py Release 1.3
# Created by Tin Tran http://gimplearn.net
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1.0: Initial release created by Tin Tran.
# Rel 1.1: Modified MrQ (add: "Rounded-Rectangle","Rectangle Distort","Ellipse", "Ellipse Distort")
# Rel 1.2: Modified MrQ (add: "Beveled", copy to new image only selected layer)
# Rel 1.3: Modified MrQ - Fixed a bug for "Ellipse Distort " (the spread was limited);
#                       - Add elements of the plugin "Pespective Border.py" created by Tin Tran
#                         https://www.gimplearn.net/viewtopic.php?f=3&t=2142&sid=bc4ab6263118fc983f39bd7e627d8a3f

from gimpfu import *
import random

def python_tt_sample_color_border(image, layer, colors, frame_percent, type, radius, concave, threshold, spread, granularity, smooth,bevel,thickness,merge):
	
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	#core code goes here/below
	active_layer = pdb.gimp_image_get_active_layer(image)
	active_layer.add_alpha()
	new_image = pdb.gimp_image_new(image.width, image.height, 0)
	display = pdb.gimp_display_new(new_image)
	active_layer = pdb.gimp_layer_new_from_drawable(layer, new_image)
	pdb.gimp_image_insert_layer(new_image, active_layer, None, 0)
	active_layer.name = layer.name

	pdb.gimp_image_convert_indexed(new_image,CONVERT_DITHER_NONE,CONVERT_PALETTE_GENERATE,colors,FALSE,TRUE,"")
	num_bytes,colormap = pdb.gimp_image_get_colormap(new_image)
	num_bytes_per_color = int(num_bytes/colors)
	pdb.gimp_context_set_sample_threshold(0.01)
	pdb.gimp_image_convert_rgb(new_image)

	#some basic pre-calculations
	frame_factor = frame_percent/100.0 + 1.0;
	new_width = layer.width * frame_factor
	new_height = layer.height * frame_factor
	total_area = new_width*new_height
	image_area = layer.width * layer.height
	border_area = total_area - image_area
	area_factor = border_area * 1.0 / image_area

	samples = []
	# process sample colors, select each color and find area and multiplied by area_factor to fit frame
	for i in range(0,int(colors)):
		startindex = i*num_bytes_per_color
		color = (colormap[startindex],colormap[startindex+1],colormap[startindex+2])
		pdb.gimp_image_select_color(new_image,CHANNEL_OP_REPLACE,active_layer,color)
		non_empty,x1,y1,x2,y2 = pdb.gimp_selection_bounds(new_image)
		if non_empty:
			mean,std_dev,median,pixels,count,percentile = pdb.gimp_drawable_histogram(active_layer,HISTOGRAM_VALUE,0,1)
			samples.append([color,count*1.0*area_factor])
	
	#sort from largest area to smallest so larger area are on the outside of frame since it'll more likely give smaller colors visibility inside
 	samples = sorted(samples, key=lambda x: -x[1])
	
	pdb.gimp_image_resize(new_image,new_width+2,new_height+2,0,0)
	new_layer = pdb.gimp_layer_new(new_image,new_image.width,new_image.height,RGBA_IMAGE,"border layer",100,LAYER_MODE_NORMAL)
	pdb.gimp_image_insert_layer(new_image,new_layer,None,0)
	current_area = total_area
	for sample in samples:
		#do work of drawing
		#calculate dimensions to select
		w = new_width * 1.0
		l = new_height * 1.0
		area = total_area * 1.0
		factor = (current_area*1.0/area)**0.5
		current_width = factor * w
		current_height = factor * l
		
		if type==0:
			pdb.gimp_image_select_rectangle(new_image,CHANNEL_OP_REPLACE,\
				new_width/2.0-(current_width/2.0),\
				new_height/2.0-(current_height/2.0),\
				current_width,current_height)
			if bevel==1:
				pdb.script_fu_add_bevel(new_image,new_layer,thickness,0,0)
				
		if type==1:
			pdb.gimp_image_select_rectangle(new_image,CHANNEL_OP_REPLACE,\
				new_width/2.0-(current_width/2.0),\
				new_height/2.0-(current_height/2.0),\
				current_width,current_height)
			pdb.script_fu_selection_rounded_rectangle(new_image,new_layer, radius, concave)
			if bevel==1:
				pdb.script_fu_add_bevel(new_image,new_layer,thickness,0,0)
				
		if type==2:
			pdb.gimp_image_select_rectangle(new_image,CHANNEL_OP_REPLACE,\
				new_width/2.0-(current_width/2.0),\
				new_height/2.0-(current_height/2.0),\
				current_width,current_height)
			pdb.script_fu_distress_selection(new_image,new_layer,threshold, spread, granularity, smooth,0,0)	
			if bevel==1:
				pdb.script_fu_add_bevel(new_image,new_layer,thickness,0,0)
				
		if type==3:
			pdb.gimp_image_select_ellipse(new_image,CHANNEL_OP_REPLACE,\
				new_width/2.0-(current_width/2.0),\
				new_height/2.0-(current_height/2.0),\
				current_width,current_height)
			if bevel==1:
				pdb.script_fu_add_bevel(new_image,new_layer,thickness,0,0)
				
		if type==4:
			pdb.gimp_image_select_ellipse(new_image,CHANNEL_OP_REPLACE,\
				new_width/2.0-(current_width/2.0),\
				new_height/2.0-(current_height/2.0),\
				current_width,current_height)
			pdb.gimp_selection_shrink(new_image, spread)
			pdb.script_fu_distress_selection(new_image,new_layer,threshold, spread, granularity, smooth,0,0)
			if bevel==1:
				pdb.script_fu_add_bevel(new_image,new_layer,thickness,0,0)
				
		pdb.gimp_edit_cut(new_layer)
		pdb.gimp_context_set_foreground(sample[0])
		pdb.gimp_edit_fill(new_layer,FILL_FOREGROUND)
		current_area -= sample[1] #subtract area for next round
	if type==5:
	
		frame_width = int(frame_percent/2.0/100.0 * layer.width)
		frame_height = int(frame_percent/2.0/100.0 * layer.height)
		min_border = min(frame_width,frame_height)
		frame_width = min_border
		frame_height = min_border
		new_width = layer.width+2*frame_width
		new_height = layer.height+2*frame_height
		pdb.gimp_image_resize(new_image,new_width,new_height,frame_width,frame_height)

		#reference points around the image
		x1,x2,y1,y2 = frame_width+1, new_width - frame_width-1, frame_height+1, new_height - frame_height-1
		pdb.gimp_selection_none(new_image)
		
		#left border
		new_layer = pdb.gimp_layer_copy(active_layer,TRUE)
		pdb.gimp_image_insert_layer(new_image,new_layer,None,0)
		pdb.gimp_item_transform_perspective(new_layer,0,0-1, x1,y1-1, 0,new_height+1, x1,y2+1)

		#right border
		new_layer = pdb.gimp_layer_copy(active_layer,TRUE)
		pdb.gimp_image_insert_layer(new_image,new_layer,None,0)
		pdb.gimp_item_transform_perspective(new_layer,x2,y1-1, new_width,0-1, x2,y2+1, new_width,new_height+1)

		#top border
		new_layer = pdb.gimp_layer_copy(active_layer,TRUE)
		pdb.gimp_image_insert_layer(new_image,new_layer,None,0)
		pdb.gimp_item_transform_perspective(new_layer,0,0, new_width,0, x1,y1, x2,y1)

		#bottom border
		new_layer = pdb.gimp_layer_copy(active_layer,TRUE)
		pdb.gimp_image_insert_layer(new_image,new_layer,None,0)
		pdb.gimp_item_transform_perspective(new_layer,x1,y2, x2,y2, 0,new_height, new_width,new_height)
		
		merged_layer = pdb.gimp_image_merge_down(new_image,new_layer,EXPAND_AS_NECESSARY)
		merged_layer = pdb.gimp_image_merge_down(new_image,merged_layer,EXPAND_AS_NECESSARY)
		merged_layer = pdb.gimp_image_merge_down(new_image,merged_layer,EXPAND_AS_NECESSARY)

		pdb.gimp_selection_none(image)
		pdb.gimp_drawable_edit_clear(active_layer)		
		new_layer = pdb.gimp_layer_new_from_drawable(layer,new_image)
		pdb.gimp_image_insert_layer(new_image,new_layer,None,0)
		pdb.gimp_layer_set_offsets(new_layer,new_width/2.0-(layer.width/2.0),new_height/2.0-(layer.height/2.0))
		
		if bevel==1:
			pdb.script_fu_add_bevel(new_image,new_layer,thickness,0,0)
			
	layer_copy = pdb.gimp_layer_new_from_drawable(layer,new_image)
	pdb.gimp_image_insert_layer(new_image,layer_copy,None,0)
	pdb.gimp_layer_set_offsets(layer_copy,new_width/2.0-(layer.width/2.0),new_height/2.0-(layer.height/2.0))
	pdb.gimp_selection_invert(new_image)
	pdb.gimp_selection_grow(new_image, num_bytes_per_color) # add MrQ for "bad pixel"
	pdb.gimp_edit_cut(layer_copy)
	pdb.gimp_selection_none(new_image)
	pdb.gimp_drawable_edit_clear(active_layer)
	pdb.gimp_layer_resize_to_image_size(active_layer)
		
	if merge==1:
		pdb.gimp_image_merge_visible_layers(new_image, 1)

	#UNCOMMENT BELOW WHEN DONE
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
    #return

register(
	"python_fu_tt_sample_color_border",                           
	" Creates Sample color border of active layer - V.1.3  ",
	"Creates Sample color border of active layer",
	"Tin Tran",
	"Tin Tran",
	"March 2019",
	"<Image>/Python-Fu/Sample Color Border...",             #Menu path
	"RGB*, GRAY*", 
	[
	(PF_SPINNER, "colors", "Number of colors(to turn to indexed to use in frame):", 10, (2, 256, 1)),
	(PF_SPINNER, "frame_percent", "Frame percentage (percent to grow by to create border):", 10, (5, 100, 1)),
	(PF_OPTION,  "type", "Type frame", 1, ["Rectangle ","Rounded-Rectangle ","Rectangle Distort ","Ellipse ", "Ellipse Distort ", "Pespective Border "]),
	(PF_SPINNER, "radius", "Rounded:\n  - Radius (%):", 1, ( 1, 100, 1, 5, 0, 0)),
	(PF_TOGGLE,  "concave", "  - Concave:", 0 ),
	(PF_SPINNER, "threshold", "Distort:\n  - Threshold (bigger 1<-->254 smaller):", 127, ( 1, 254, 1, 5, 0, 0)),
	(PF_SPINNER, "spread", "  - Spread:" , 8, ( 1, 20, 1, 5, 0, 0)),
	(PF_SPINNER, "granularity", "  - Granularity (1 is low):", 4, ( 1, 25, 1, 5, 0, 0)),
	(PF_SPINNER, "smooth", "  - Smooth:", 2, ( 1, 150, 1, 5, 0, 0)),
	(PF_TOGGLE,  "bevel", "Beveled:", 0 ),
	(PF_SPINNER, "thickness", "  - Bevel - Thickness:", 5, ( 0, 30, 1, 5, 0, 0)),
	(PF_TOGGLE,  "merge", "Merge:", 1 ),	
	],
	[],
	python_tt_sample_color_border)

main()